function res = pnmftest_purity_entropy(method, dataname)
switch dataname
    case 'orl';
        load orl.mat;
        X = double(X); X = X/max(max(X));
    case 'optdigits2460'
        load optdigits2460.mat;
    case 'iris'
        load iris.mat;
    otherwise
        error('data not found');
end

classes = clabel2dataclasses(C, nc);

ntry = 100;

res = init_res(ntry);

maxiter = 10000;
r = nc;
verbose = false;

switch method
    case 'kmeans'
        for i=1:ntry
            fprintf('kmeans: i=%d\n', i);
            IDX = kmeans(X, nc);
            clusters = clabel2dataclasses(IDX,nc);
            [res.purity(i), res.entropy(i)] = ...
                ComputeClusterPerf(clusters', classes);
        end
    case 'nmf'
        for i=1:ntry
            fprintf('nmf: i=%d\n', i);
            [W,~] = nmfeu(X,r,maxiter, i, verbose);
            clusters = W';
            [res.purity(i), res.entropy(i)] = ...
                ComputeClusterPerf(clusters, classes);
        end
    case 'onmf'
        for i=1:ntry
            fprintf('onmf: i=%d\n', i);
            [L,~] = onmfeu(X, r, maxiter, i, verbose);
            clusters = L;
            [res.purity(i), res.entropy(i)] = ...
                ComputeClusterPerf(clusters', classes);
        end
    case 'pnmf'
        for i=1:ntry
            fprintf('pnmf: i=%d\n', i);
            L = pnmfeu(X, r, maxiter, i, verbose);
            clusters = L;
            [res.purity(i), res.entropy(i)] = ...
                ComputeClusterPerf(clusters', classes);
        end
    case 'cvxnmf'
        for i=1:ntry
            fprintf('cvxnmf: i=%d\n', i);
            [~, G] = cvxnmfeu(X, r, maxiter, i, verbose);
            clusters = G;
            [res.purity(i), res.entropy(i)] = ...
                ComputeClusterPerf(clusters', classes);
        end
end

function res = init_res(ntry)
res.purity = zeros(ntry,1);
res.entropy = zeros(ntry,1);

function dataclasses = clabel2dataclasses(clabel, nc)
n = length(clabel);
dataclasses = sparse((1:n)', clabel(:), ones(n,1), n, nc);

function [purity, entropy] = ComputeClusterPerf(H, classes)
[~,n] = size(H);
nc = size(classes,2);

clusters = zeros(size(H));
[~, ind] = max(H);
clusters(sub2ind(size(clusters), ind, 1:n)) = 1;

T = clusters * classes;

purity = sum(max(T, [], 2)) / n;

sc = sum(T,2);
indnz = sc>0;
T = T(indnz,:);
tmp = bsxfun(@rdivide, T, sc(indnz)+eps);
indnz = tmp>0;
entropy = -sum(T(indnz).*log2(tmp(indnz)+eps)) / (n*log2(nc));

function [W,H] = nmfeu(X,r,maxiter, nSeed, verbose)
[m,n] = size(X);

rand('seed', nSeed);
W = rand(m,r);
H = rand(r,n);

for iter=1:maxiter
    if verbose
        fprintf('iter=%d, ', iter);
    end
    
    H = H .* (W'*X) ./ (W'*W*H+eps);
    W = W .* (X*H') ./ (W*(H*H')+eps);
    W = bsxfun(@rdivide, W, sum(W)+eps);
    
    if verbose
        fprintf('\n');
    end
end

function [L,R] = onmfeu(X, r, maxiter, nSeed, verbose)
[m,n] = size(X);

rand('seed', nSeed);
L = rand(m,r);
R = rand(n,r);

for iter=1:maxiter
    if verbose
        fprintf('iter=%d,', iter);
    end
    
    R = R .* ( X' * L ) ./ (R * (L' * L));
    L = L .* ( X * R ) ./ (L * (L' * X * R));
    L = L .* sqrt(trace(L'*X*R)/trace(L'*L*(R'*R)));
    
    if verbose
        fprintf('\n');
    end
end

function W = pnmfeu(X, r, maxiter, nSeed, verbose)
[m,n] = size(X);

rand('seed', nSeed);
W = rand(m,r);

bUseCorr = m<n && m<2000;

if bUseCorr
    XX = X * X';
end

for iter=1:maxiter
    if verbose
        fprintf('iter=%d, ', iter);
    end
    
    if bUseCorr
        W = W .* (XX*W) ./ (W*(W'*XX*W) + XX*W*(W'*W) + eps);
        WXXW = W'*XX*W;
    else
        W = W .* (X*(X'*W)) ./ (W*(W'*X*(X'*W)) + X*(X'*W)*(W'*W) +eps);
        WX = W' * X;
        WXXW = WX*WX';
    end
    W = W .* sqrt(trace(WXXW)./trace(WXXW*(W'*W)));
    
    if verbose
        fprintf('\n');
    end
end

function [W, G] = cvxnmfeu(X, r, maxiter, nSeed, verbose)
[m,n] = size(X);

rand('seed', nSeed);
W = rand(m,r);
G = rand(m,r);

bUseCorr = m<2000 && m<n;

if bUseCorr
    XX = X * X';
end

eps = 1e-9;

for i=1:maxiter
    if verbose
        fprintf('iter=%d ', i);
    end
    
    if bUseCorr
        G = G .* sqrt(XX*W./(G*(W'*XX*W)+eps));
        W = W .* sqrt(XX*G./(XX*W*(G'*G)+eps));
    else
        XW = X' * W;
        G = G .* sqrt((X*XW)./(G*(XW'*XW)+eps));
        W = W .* sqrt((X*(X'*G))./(X*(XW)*(G'*G)+eps));
    end
                
    if verbose
        fprintf('\n');
    end
end
